@@ -253,12 +253,36 @@ export function addActionEventListener(component, eventType) {
 /**
  * Adds a model event listener to the element.
  * @param {Component} component Component that contains the element.
- * @param {DOM Element} el DOM Element that will get the event attached.
- * @param {string} eventType Event type to listen for.
+ * @param {Element} Element that will get the event attached.
+ * @param {string} eventType Event type to listen for. Optional; will use `model.eventType` by default.
  */
-export function addModelEventListener(component, el, eventType) {
+export function addModelEventListener(component, element, eventType) {
+  eventType = eventType || element.model.eventType;
+  element.events.push(eventType);
+  const { el } = element;
+
   el.addEventListener(eventType, (event) => {
-    const element = new Element(event.target);
+    let isDirty = false;
+
+    if (component.data[element.model.name] !== element.getValue()) {
+      isDirty = true;
+      element.handleDirty();
+    } else {
+      element.handleDirty(true);
+    }
+
+    if (element.model.isLazy) {
+      // Lazy models fire an input and blur so that the dirty check above works as expected.
+      // This will prevent the input event from doing anything.
+      if (eventType === "input") {
+        return;
+      }
+
+      // Lazy non-dirty elements can bail
+      if (!isDirty) {
+        return;
+      }
+    }
 
     const action = {
       type: "syncInput",
@@ -273,21 +297,26 @@ export function addModelEventListener(component, el, eventType) {
     }
 
     if (element.model.isDefer) {
-      let foundAction = false;
+      let foundActionIdx = -1;
 
       // Update the existing action with the current value
-      component.actionQueue.forEach((a) => {
+      component.actionQueue.forEach((a, idx) => {
         if (a.payload.name === element.model.name) {
           a.payload.value = element.getValue();
-          foundAction = true;
+          foundActionIdx = idx;
         }
       });
 
       // Add a new action
-      if (!foundAction) {
+      if (isDirty && foundActionIdx === -1) {
         component.actionQueue.push(action);
       }
 
+      // Remove the found action that isn't dirty
+      if (!isDirty && foundActionIdx > -1) {
+        component.actionQueue.splice(foundActionIdx);
+      }
+
       return;
     }
 
@@ -318,13 +347,15 @@ export function addModelEventListener(component, el, eventType) {
 /**
  * Adds a db event listener to the element.
  * @param {Component} component Component that contains the element.
- * @param {DOM Element} el DOM `Element` that will get the event attached.
- * @param {string} eventType Event type to listen for.
+ * @param {Element} Element that will get the event attached.
+ * @param {string} eventType Event type to listen for. Optional; will use `field.eventType` by default.
  */
-export function addDbEventListener(component, el, eventType) {
-  el.addEventListener(eventType, (event) => {
-    const element = new Element(event.target);
+export function addDbEventListener(component, element, eventType) {
+  eventType = eventType || element.field.eventType;
+  element.events.push(eventType);
+  const { el } = element;
 
+  el.addEventListener(eventType, (event) => {
     if (
       (isEmpty(element.db.name) && isEmpty(element.model.name)) ||
       isEmpty(element.db.pk)
@@ -332,6 +363,33 @@ export function addDbEventListener(component, el, eventType) {
       return;
     }
 
+    let isDirty = false;
+
+    for (let i = 0; i < component.data[element.model.name].length; i++) {
+      const dbModel = component.data[element.model.name][i];
+
+      if (dbModel.pk.toString() === element.db.pk) {
+        if (dbModel[element.field.name] !== element.getValue()) {
+          element.handleDirty();
+          isDirty = true;
+        } else {
+          element.handleDirty(true);
+        }
+      }
+    }
+
+    if (element.field.isLazy) {
+      // Lazy models fire an input and blur so that the dirty check above works as expected.
+      // This will prevent the input event from doing anything.
+      if (eventType === "input") {
+        return;
+      }
+      // Lazy non-dirty elements can bail
+      if (!isDirty) {
+        return;
+      }
+    }
+
     if (!component.lastTriggeringElements.some((e) => e.isSame(element))) {
       component.lastTriggeringElements.push(element);
     }
@@ -348,21 +406,26 @@ export function addDbEventListener(component, el, eventType) {
     action.payload.fields[element.field.name] = element.getValue();
 
     if (element.field.isDefer) {
-      let foundAction = false;
+      let foundActionIdx = -1;
 
       // Update the existing action with the current value
-      component.actionQueue.forEach((a) => {
+      component.actionQueue.forEach((a, idx) => {
         if (generateDbKey(a.payload) === element.dbKey()) {
           a.payload.fields[element.field.name] = element.getValue();
-          foundAction = true;
+          foundActionIdx = idx;
         }
       });
 
       // Add a new action
-      if (!foundAction) {
+      if (isDirty && foundActionIdx === -1) {
         component.actionQueue.push(action);
       }
 
+      // Remove the found action that isn't dirty
+      if (!isDirty && foundActionIdx > -1) {
+        component.actionQueue.splice(foundActionIdx);
+      }
+
       return;
     }
 
