@@ -8,11 +8,14 @@ from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.http import require_POST
 
 import orjson
+from bs4 import BeautifulSoup
 
 from .call_method_parser import InvalidKwarg, parse_call_method_name, parse_kwarg
 from .components import UnicornField, UnicornView
+from .decorators import timed
 from .errors import UnicornViewError
 from .message import ComponentRequest, Return
+from .serializer import loads
 
 
 logger = logging.getLogger(__name__)
@@ -31,6 +34,7 @@ def handle_error(view_func):
     return wraps(view_func)(wrapped_view)
 
 
+@timed
 def _set_property_from_data(
     component_or_field: Union[UnicornView, UnicornField, Model], name: str, value,
 ) -> None:
@@ -57,6 +61,7 @@ def _set_property_from_data(
                 setattr(component_or_field, name, value)
 
 
+@timed
 def _set_property_value(
     component: UnicornView, property_name: str, property_value: Any, data: Dict = {}
 ) -> None:
@@ -128,6 +133,7 @@ def _set_property_value(
     component.updated(property_name, property_value)
 
 
+@timed
 def _get_property_value(component: UnicornView, property_name: str) -> Any:
     """
     Gets property value from the component based on the property name.
@@ -157,6 +163,7 @@ def _get_property_value(component: UnicornView, property_name: str) -> Any:
                 component_or_field = component_or_field[property_name_part]
 
 
+@timed
 def _call_method_name(
     component: UnicornView, method_name: str, params: List[Any]
 ) -> Any:
@@ -178,6 +185,7 @@ def _call_method_name(
             return func()
 
 
+@timed
 @handle_error
 @csrf_protect
 @require_POST
@@ -381,4 +389,36 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
                 {"poll": return_data.poll,}
             )
 
+    parent_component = component.parent
+
+    if parent_component:
+        parent_frontend_context_variables = (
+            parent_component.get_frontend_context_variables()
+        )
+
+        dom = parent_component.render()
+
+        soup = BeautifulSoup(dom, features="html.parser")
+        checksum = None
+
+        # TODO: This doesn't create the same checksum for some reason
+        # checksum = orjson.dumps(parent_frontend_context_variables)
+
+        for element in soup.find_all():
+            if "unicorn:checksum" in element.attrs:
+                checksum = element["unicorn:checksum"]
+                break
+
+        res.update(
+            {
+                "parent": {
+                    "id": parent_component.component_id,
+                    "dom": dom,
+                    "checksum": checksum,
+                    "data": loads(parent_frontend_context_variables),
+                    "errors": parent_component.errors,
+                }
+            }
+        )
+
     return JsonResponse(res)
